/*******************************************************************************
 * Instance třídy {@code KomentovanyStrom} představují ukázkovou třídu 
 * pro výklad komentářů
 *
 * @author    Pavel Herout
 * @version   2.00.000
 */
public class KomentovanyStrom
{
//== KONSTANTNÍ ATRIBUTY TŘÍDY =================================================

  /** standardní barva kmene stromu */
  private static final Barva BARVA_KMENE = Barva.HNEDA;

  /** implicitní barva koruny stromu */
  private static final Barva IMPL_BARVA_KORUNY = Barva.ZELENA;

  /** barva koruny stromu na podzim */
  private static final Barva BARVA_PODZIMU = Barva.ZLUTA;


//== PROMĚNNÉ ATRIBUTY TŘÍDY ===================================================

  /** Počet doposud vytvořených instancí. */
  private static int pocet = 0;
  
  
//== STATICKÝ INICIALIZAČNÍ BLOK - STATICKÝ KONSTRUKTOR ========================
//== KONSTANTNÍ ATRIBUTY INSTANCÍ ==============================================

  /** Pořadí kolikátá byla daná instance vytvořena v rámci třídy. */
  private final int PORADI = ++pocet;

  /** Název instance sestávající z názvu třídy a POŘADÍ instance */
  private final String NAZEV  = "Strom_" + PORADI;

  /** koruna stromu */
  private final Elipsa koruna;
  /** kmen stromu */
  private final Obdelnik kmen;

//== PROMĚNNÉ ATRIBUTY INSTANCÍ ================================================

  /** x souřadnice levého horního rohu stromu
   *  ve skutečnosti zbytečná, je použita jen pro ukázku {@code this}
   */
  private int x;
  
//== PŘÍSTUPOVÉ METODY VLASTNOSTÍ TŘÍDY ========================================
//== OSTATNÍ NESOUKROMÉ METODY TŘÍDY ===========================================

//##############################################################################
//== KONSTRUKTORY A TOVÁRNÍ METODY =============================================

  /***************************************************************************
   * Implicitní konstruktor vytvoří v levém horním rohu plátna
   * instanci širokou 100 bodů, vysokou 150 bodů
   * s {@code BARVA_KMENE} a {@code IMPL_BARVA_KORUNY}
   */
  public KomentovanyStrom() {
    this(0, 0);
  }

  /***************************************************************************
   * Vytvoří na definovaných souřadnicích (levý horní roh)
   * instanci širokou 100 bodů, vysokou 150 bodů
   * s {@code BARVA_KMENE} a {@code IMPL_BARVA_KORUNY}
   * 
   * @param x  x-ová souřadnice instance, x>=0, x=0 má levý okraj plátna
   * @param y  y-ová souřadnice instance, y>=0, y=0 má horní okraj plátna
   */
  public KomentovanyStrom(int x, int y) {
    this(x, y, IMPL_BARVA_KORUNY);
  }

  /***************************************************************************
   * Vytvoří na definovaných souřadnicích (levý horní roh)
   * instanci širokou 100 bodů, vysokou 150 bodů
   * s {@code BARVA_KMENE} a zadanou barvou koruny
   * jako vedlejší efekt uloží x-souřadnici - pouze pro ukázku použití {@code this}
   * 
   * @param x  x-ová souřadnice instance, x>=0, x=0 má levý okraj plátna
   * @param y  y-ová souřadnice instance, y>=0, y=0 má horní okraj plátna
   * @param barvaKoruny  barva koruny
   */
  public KomentovanyStrom(int x, int y, Barva barvaKoruny) {
    koruna = new Elipsa(x, y, 100, 100, barvaKoruny);
    kmen = new Obdelnik(x + 45, y + 100, 10, 50, BARVA_KMENE);
    this.x = x;
  }


//== ABSTRAKTNÍ METODY =========================================================
//== PŘÍSTUPOVÉ METODY VLASTNOSTÍ INSTANCÍ =====================================
  /***************************************************************************
   * Vrátí x-ovou souřadnici pozice instance.
   *
   * @return  x-ová souřadnice.
   */
  public int getX() {
    return this.x;  
  }  


  /***************************************************************************
   * Vrátí y-ovou souřadnici pozice instance.
   *
   * @return  y-ová souřadnice.
   */
  public int getY() {
    return koruna.getY();  
  }  

  /***************************************************************************
   * Vrátí název instance, tj. název její třídy následovaný POŘADÍm.
   *
   * @return  Řetězec s názvem instance.
   */
  public String getNazev(){
    return NAZEV;
  }

  /***************************************************************************
   * Nastaví novou barvu koruny.
   *
   * @param nová  Požadovaná nová barva.
   */
  public void setBarvaKoruny(Barva barva) {
    koruna.setBarva(barva);
  }

  /***************************************************************************
   * Vrátí výšku instance.
   *
   * @return  Výška instance v bodech
   */
  public int getVyska() {
    return koruna.getVyska() + kmen.getVyska();  
  }  

  /***************************************************************************
   * Vrátí informaci o tom, zda má koruna podzimní korunu.
   * Pojmenování je nevhodné - ukázka chybné analýzy.
   *
   * @return  {@code true}, pokud má koruna podzimní barvu, jinak {@code false}
   */
  public boolean isPodzim() {
//     return koruna.getBarva() == BARVA_PODZIMU;

//     Barva barva = koruna.getBarva();
//     boolean jePodzimni = barva.equals(BARVA_PODZIMU);
//     return jePodzimni;
    return koruna.getBarva().equals(BARVA_PODZIMU);
  }
  
//== OSTATNÍ NESOUKROMÉ METODY INSTANCÍ ========================================

  /***************************************************************************
   * Změní barvu koruny na podzimní barvu.
   */
  public void zezloutni() {
    setBarvaKoruny(BARVA_PODZIMU);
  }
  
  /***************************************************************************
   * Změní postupně barvu koruny na podzimní barvu
   * <br/>
   * nejprve použije KHAKI barvu a pak po půl vteřině finální ZLUTA
   */
  public void postupneZezloutni() {
    setBarvaKoruny(Barva.KHAKI);
    IO.cekej(500);
    setBarvaKoruny(BARVA_PODZIMU);
  }

  /***************************************************************************
   * Vykreslí obraz své instance na plátno.
   */
  public void nakresli() {
    koruna.nakresli();
    kmen.nakresli();
  }
  
  /***************************************************************************
   * Smaže obraz své instance z plátna.
   */
  public void smaz() {
    koruna.smaz();
    kmen.smaz();
  }

  /***************************************************************************
   * Převede instanci na řetězec obsahující název třídy, POŘADÍ instance,
   * její souřadnice, výšku a barvu koruny.
   *
   * @return Řetězcová reprezentace dané instance.
   */
  @Override
  public String toString() {
    return getNazev() + ": x=" + getX() + ", y=" + getY() +
             ", výška=" + getVyska() + 
             ", barva=" + koruna.getBarva().getNazev();
  }

  
//== SOUKROMÉ A POMOCNÉ METODY TŘÍDY ===========================================
//== SOUKROMÉ A POMOCNÉ METODY INSTANCÍ ========================================
//== INTERNÍ DATOVÉ TYPY =======================================================
//== TESTY A METODA MAIN =======================================================
//
//     /***************************************************************************
//      * Testovací metoda.
//      */
//     public static void test()
//     {
//         KomentovanyStrom instance = new KomentovanyStrom();
//         System.out.println(instance);
//     }
//     /** @param args Parametry příkazového řádku - nepoužívané. */
//     public static void main(String[] args)  {  test();  }
}
