/***************************************************************
 * Instance třídy {@code Ctverec} představují čtverce
 *
 * @author    Pavel Herout
 * @version   2.00.000
 */
public class Ctverec implements IKresleny, IPosuvny {

//== KONSTANTNÍ ATRIBUTY TŘÍDY =================================================
  /** Implicitní rozměr */
  private static final int IMPLICITNI_ROZMER = 100;

  /** Počáteční barva nakreslené instance v případě,
   *  kdy uživatel žádnou požadovanou barvu nezadá 
   */
  public static final Barva IMPLICITNI_BARVA = Barva.ZLUTA;

//== KONSTANTNÍ ATRIBUTY INSTANCÍ =============================
  /** skutečná reprezentace pomocí již existujícího tvaru */
  private final Obdelnik obdelnik;

//#############################################################
//== KONSTRUKTORY A TOVÁRNÍ METODY ============================

  /***************************************************************************
   * Vytvoří novou instanci se polohou [0, 0] a rozměrem 100
   * a implicitní barvou.
   */
  public Ctverec() {
    this(0, 0, IMPLICITNI_ROZMER);
  }

  /***************************************************************************
   * Vytvoří novou instanci se zadanou polohou a rozměrem
   * a implicitní barvou.
   *
   * @param x       x-ová souřadnice instance, x>=0, x=0 má levý okraj plátna
   * @param y       y-ová souřadnice instance, y>=0, y=0 má horní okraj plátna
   * @param strana  velikost strany vytvářené instance, strana >= 0
   */
  public Ctverec(int x, int y, int strana) {
    this( x, y, strana, IMPLICITNI_BARVA );
  }
  
  /***************************************************************************
   * Vytvoří novou instanci se zadanými rozměrem, polohou a barvou.
   *
   * @param x       x-ová souřadnice instance, x>=0, x=0 má levý okraj plátna
   * @param y       y-ová souřadnice instance, y>=0, y=0 má horní okraj plátna
   * @param strana  velikost strany vytvářené instance, strana >= 0
   * @param barva   Barva vytvářené instance
   */
  public Ctverec(int x, int y, int strana, Barva barva) {
    obdelnik = new Obdelnik(x, y, strana, strana, barva);
  }
  
//== PŘÍSTUPOVÉ METODY VLASTNOSTÍ INSTANCÍ ====================
  /***************************************************************************
   * Vrátí x-ovou souřadnici pozice instance.
   *
   * @return  x-ová souřadnice.
   */
  public int getX() {
    return obdelnik.getX();
  }

  /***************************************************************************
   * Vrátí y-ovou souřadnici pozice instance.
   *
   * @return  y-ová souřadnice.
   */
  public int getY() {
    return obdelnik.getY();
  }

  /***************************************************************************
   * Vrátí instanci třídy Pozice s pozicí instance.
   *
   * @return   Pozice s pozicí instance.
   */
  public Pozice getPozice() {
    return new Pozice(getX(), getY());
  }


  /***************************************************************************
   * Nastaví novou pozici instance.
   *
   * @param x   Nová x-ová pozice instance
   * @param y   Nová y-ová pozice instance
   */
  public void setPozice(int x, int y) {
    obdelnik.setPozice(new Pozice(x, y));
  }

  /***************************************************************************
   * Nastaví novou pozici instance.
   *
   * @param pozice   Nová pozice instance
   */
  public void setPozice(Pozice pozice) {
    obdelnik.setPozice(pozice.x, pozice.y);
  }


  /***************************************************************************
   * Vrátí velikost strany instance.
   *
   * @return  Velikost strany v bodech
   */
  public int getStrana() {
    return obdelnik.getSirka();
  }

  /***************************************************************************
   * Nastaví velikost strany instance.
   *
   * @param strana  Velikost strany v bodech
   */
  public void setStrana(int strana) {
    obdelnik.setRozmer(strana, strana);
  }

  /***************************************************************************
   * Vrátí barvu instance.
   *
   * @return  Instance třídy Barva definující nastavenou barvu.
   */
  public Barva getBarva() {
    return obdelnik.getBarva();
  }


  /***************************************************************************
   * Nastaví novou barvu instance.
   *
   * @param nova   Požadovaná nová barva.
   */
  public void setBarva(Barva nova) {
     obdelnik.setBarva(nova);
  }
   
//== PŘEKRYTÉ METODY IMPLEMENTOVANÝCH ROZHRANÍ =================================

  /***************************************************************************
   * Za pomoci dodaného kreslítka vykreslí obraz své instance
   * na animační plátno.
   *  
   * @param kreslitko   Kreslitko, kterým se instance nakreslí na plátno.     
   */
  public void nakresli(Kreslitko kreslitko) {
    obdelnik.nakresli(kreslitko);
  }

//== NOVĚ ZAVEDENÉ METODY INSTANCÍ =============================================

  /***************************************************************************
   * Přesune instanci o zadaný počet bodů vpravo,
   * při záporné hodnotě parametru vlevo.
   *
   * @param vzdalenost Vzdálenost, o kterou se instance přesune.
   */
  public void posunVpravo(int vzdalenost) {
    obdelnik.posunVpravo(vzdalenost);
  }

//== PŘEKRYTÉ KONKRÉTNÍ METODY RODIČOVSKÉ TŘÍDY ================================
  /***************************************************************************
   * Převede instanci na řetězec. Používá se především při ladění.
   * 
   * @return Řetězcová reprezentace dané instance.      
   */
  @Override
  public String toString()
  {
    return this.getClass().getSimpleName() + 
             ": x=" + obdelnik.getX() + ", y=" + obdelnik.getY() + 
             ", strana=" + obdelnik.getSirka() +
             ", barva=" + obdelnik.getBarva();
  }


//== TESTOVACÍ METODY A TŘÍDY =================================
//
//   /*******************************************************
//    * Testovací metoda.
//    */
//   public static void test() {
//     Ctverec instance = new Ctverec();
//   }
//   /** @param args Parametry příkazového řádku - nepoužívané. */
//   public static void main(String[] args)  {  test();  }
}
