/***************************************************************
 * Instance třídy {@code ZdedenyCtverec} představují čtverce
 *
 * @author    Pavel Herout
 * @version   2.00.000
 */
public final class ZdedenyCtverec extends Obdelnik {

//== KONSTANTNÍ ATRIBUTY TŘÍDY =================================================
  /** Implicitní rozměr */
  private static final int IMPLICITNI_ROZMER = 100;

  /** Počáteční barva nakreslené instance v případě,
   *  kdy uživatel žádnou požadovanou barvu nezadá 
   */
  public static final Barva IMPLICITNI_BARVA = Barva.ZLUTA;

//#############################################################
//== KONSTRUKTORY A TOVÁRNÍ METODY ============================

  /*************************************************************
   * Vytvoří novou instanci se polohou [0, 0] a rozměrem 100
   * a implicitní barvou.
   */
  public ZdedenyCtverec() {
    this(0, 0, IMPLICITNI_ROZMER);
  }

  /***************************************************************
   * Vytvoří novou instanci se zadanou polohou a rozměrem
   * a implicitní barvou.
   *
   * @param x       x-ová souřadnice instance, x>=0, x=0 má levý okraj plátna
   * @param y       y-ová souřadnice instance, y>=0, y=0 má horní okraj plátna
   * @param strana  velikost strany vytvářené instance, strana >= 0
   */
  public ZdedenyCtverec(int x, int y, int strana) {
    this( x, y, strana, IMPLICITNI_BARVA );
  }
  
  /***************************************************************
   * Vytvoří novou instanci se zadanými rozměrem, polohou a barvou.
   *
   * @param x       x-ová souřadnice instance, x>=0, x=0 má levý okraj plátna
   * @param y       y-ová souřadnice instance, y>=0, y=0 má horní okraj plátna
   * @param strana  velikost strany vytvářené instance, strana >= 0
   * @param barva   Barva vytvářené instance
   */
  public ZdedenyCtverec(int x, int y, int strana, Barva barva) {
    super(x, y, strana, strana, barva);
  }
  
//== PŘÍSTUPOVÉ METODY ATRIBUTU INSTANCÍ ============================
  /******************************************************************
   * Nastaví nové rozměry instance - oba parametry by měly být stejné.
   * Při rozdílných hodnotách se použije menší z nich.
   * Metoda musí být překryta, aby byl zajištěn kontrakt, že čtverec má
   * stejně dlouhé strany
   *
   * @param sirka    Nově nastavovaná šířka; šířka>=0
   * @param vyska    Nově nastavovaná výška; výška>=0
   */
  @Override
  public void setRozmer(int sirka, int vyska) {
    int strana = Math.min(sirka, vyska);
    super.setRozmer(strana, strana);
//     super.setRozmer(Math.min(sirka, vyska));
  }  
  
//   /******************************************************************
//    * Chybná verze - zacyklené volání
//    *
//    * @param šířka    Nově nastavovaná šířka; šířka>=0
//    * @param výška    Nově nastavovaná výška; výška>=0
//    */
//   @Override
//   public void setRozmer(int sirka, int vyska) {
//     super.setRozmer(Math.min(sirka, vyska));
//   }  
  
}
