/**************************************************************
 * Instance třídy {@code ADopravniZnacka} představují dopravní značky
 *
 * @author    Pavel Herout
 * @version   2.00.000
 */
public abstract class ADopravniZnacka implements IKresleny {

//== KONSTANTNÍ ATRIBUTY TŘÍDY ================================
  /** Barva sloupku */
  public static final Barva BARVA_SLOUPKU = Barva.SEDA;

  /** Aktivní plátno, které dohlíží na správné vykreslení instance. */
  private static final SpravcePlatna SP = SpravcePlatna.getInstance();

//== KONSTANTNÍ ATRIBUTY INSTANCÍ =============================
  private final Obdelnik sloupek;

//== PROMĚNNÉ ATRIBUTY INSTANCÍ ================================================
  private IKresleny cedule = null;

//== KONSTRUKTORY A TOVÁRNÍ METODY ============================
  /**********************************************************
   * vytvoří sloupek pod dopravní značkou a pak příslušnou ceduli
   * obě části zaregistruje u správce plátna
   * @param x x-souřadnice
   * @param y y-souřadnice
   */
  public ADopravniZnacka(int x, int y) {
    sloupek = new Obdelnik(x + 40, y + 100, 20, 100, BARVA_SLOUPKU);
    SP.pridej(sloupek);
    cedule = vytvorCeduli(x, y);
    SP.pridej(cedule);
  }

//== ABSTRAKTNÍ METODY ========================================
  /************************************************************
   * Vytvoří ceduli správného tvaru na požadovaném místě
   * @param x x-souřadnice
   * @param y y-souřadnice
   * @returns objekt cedule
   */
  protected abstract IKresleny vytvorCeduli(int x, int y);
  
//== PŘEKRYTÉ METODY IMPLEMENTOVANÝCH ROZHRANÍ ================
  /************************************************************
   * Za pomoci dodaného kreslítka vykreslí obraz své instance
   * na animační plátno.
   *  
   * @param kreslítko   Kreslítko, kterým se instance nakreslí na plátno.     
   */
  @Override
  public void nakresli(Kreslitko kreslitko) {
    sloupek.nakresli(kreslitko);
    cedule.nakresli(kreslitko);
  }
}
